import {Injectable, OnDestroy} from '@angular/core';
import {HttpClient} from '@angular/common/http';
import {environment} from '../../environments/environment';
import {filter, map, shareReplay} from 'rxjs/operators';
import {combineLatest, merge, Observable, Subject, Subscription} from 'rxjs';
import {getLocalStorageRole, setLocalStorageRole} from '../../utils/localstorage';

export interface UserContext {
  commonName: string;
  organizationName: string;
  eligibleRoles: UserAuthorizationType[];
}

export interface UserAuthorizationType {
  authorizationCode: string;
  educationCode: string;
  educationType: string;
}

@Injectable({
  providedIn: 'root'
})
export class UserContextService implements OnDestroy{

  constructor(private http: HttpClient) {
    // Subscribe to make to make the observable hot so that any previous role selected will be fetched
    this.fetchSelectedRoleSubscription = this.userContext$.pipe(
      map(userContext => userContext.commonName)
    ).subscribe(
      username => {
        const lastSelectedRole = getLocalStorageRole(username);
        if (lastSelectedRole) {
          this.selectRole(lastSelectedRole);
        }
      }
    );

    // Subscribe to make to make the observable hot so local storage is updated upon role selection
    this.updateLocalStorageSubscription = this.localStorageKeyValue$.subscribe(
    ([role, user]) =>
      setLocalStorageRole(user, role)
    );
  }
  private updateLocalStorageSubscription: Subscription;
  private fetchSelectedRoleSubscription: Subscription;
  private USERCONTEXT_API = '/usercontext';
  private LOGOUT_PATH = '/saml/Logout';

  public userContext$ = this.http.get<UserContext>(environment.base_path + '/api' + this.USERCONTEXT_API).pipe(
    shareReplay()
  );
  public eligibleRoles$ = this.userContext$.pipe(
    map(userContext => userContext.eligibleRoles),
    shareReplay()
  );
  private innerSelectedRole: Subject<UserAuthorizationType> = new Subject();
  public selectedRole$: Observable<UserAuthorizationType> = merge(
      this.innerSelectedRole,
      this.eligibleRoles$.pipe(
        filter(eligibleRoles => eligibleRoles.length === 1),
        map(eligibleRoles => {
          // Auto select when only one role is eligible
           return eligibleRoles[0];
        }))
  ).pipe(
    shareReplay()
  );

  public localStorageKeyValue$ = combineLatest([
    this.selectedRole$.pipe(),
    this.userContext$.pipe(map(userContext => userContext.commonName)),
  ]);

  public logout(): void {
    console.log('Logout triggered');

    window.location.assign(environment.base_path + this.LOGOUT_PATH);
  }

  public selectRole(role: UserAuthorizationType): void {
    console.log('UserContextService selectRole() : ' + JSON.stringify(role));
    this.innerSelectedRole.next(role);
  }

  ngOnDestroy(): void {
    this.fetchSelectedRoleSubscription.unsubscribe();
    this.updateLocalStorageSubscription.unsubscribe();
  }
}
