import {HttpErrorResponse} from '@angular/common/http';
import {Observable, throwError} from 'rxjs';
import {environment} from '../environments/environment';
import {deleteCookie} from './cookies';


/**
 * Specific app error handler
 * @param error HttpErrorResponse
 */
export function handleError(error: HttpErrorResponse): Observable<never> {
  console.log('handleError: status: ' + error.status + ' message: ' + error.message);
  redirectIfRequested(error);
  if (error.status === 404) {
    return throwError(new PageNotFoundError(error));
  }
  return throwError(new AppError(error));
}

/**
 * Browser intercepts redirects in xhr calls and returns 200 OK
 * this will check if any login redirects were requested by SAML
 * @param error the httpErrorResponse
 */
function redirectIfRequested(error: HttpErrorResponse): void {
  const redirectHtmlStart = '<html><head><meta http-equiv="refresh" content=';
  // Regex for oiosaml.fragment Cookie: url=(.*?)\".*document\.cookie = '(.*)';

  if (error.status === 200 && error.error.text?.includes(redirectHtmlStart)) {
    console.log('should be redirected');
    const redirectUrlRegex = /url=(.*?)\"/;
    const matches = redirectUrlRegex.exec(error.error.text);
    if (matches !== null && matches.length > 1) {
      deleteCookie('JSESSIONID');
      window.location.assign(environment.base_path);
    }
  }
}

// Custom App Errors
/**
 * The Generic Application error. Parent to all other errors.
 */
export interface GenericAppError {
  errorMessage: string;
  innerError?: Error;
}

/**
 * Top level error
 */
export class AppError implements GenericAppError {
  errorMessage = 'Der skete en ukendt fejl';
  constructor(public innerError?: Error) {
  }
}

/**
 * Expected error for 404
 */
export class PageNotFoundError extends AppError {
  errorMessage = 'Siden kunne ikke findes';
}
